@@ -1,8 +1,14 @@
-package SVN::Hooks::CheckJira;
-
 use strict;
 use warnings;
+
+package SVN::Hooks::CheckJira;
+{
+  $SVN::Hooks::CheckJira::VERSION = '1.19';
+}
+# ABSTRACT: Integrate Subversion with the JIRA ticketing system.
+
 use Carp;
+use Data::Util qw(:check);
 use SVN::Hooks;
 use JIRA::Client;
 
@@ -10,12 +16,231 @@ use Exporter qw/import/;
 my $HOOK = 'CHECK_JIRA';
 our @EXPORT = qw/CHECK_JIRA_CONFIG CHECK_JIRA/;
 
-our $VERSION = $SVN::Hooks::VERSION;
+
+my ($BaseURL, $Login, $Passwd, $MatchLog, $MatchKey);
+my $JIRA;
+my @Checks;
+my %Defaults = (
+    require     => 1,
+    valid       => 1,
+    unresolved  => 1,
+    by_assignee => 0,
+);
+
+sub CHECK_JIRA_CONFIG {
+    ($BaseURL, $Login, $Passwd, $MatchLog, $MatchKey) = @_;
+
+    if (defined $MatchKey) {
+	is_rx($MatchKey) or croak "CHECK_JIRA_CONFIG: fifth argument must be a Regexp.\n";
+    } else {
+	$MatchKey = qr/[A-Z]{2,}/;
+    }
+
+    if (defined $MatchLog) {
+	is_rx($MatchLog) or croak "CHECK_JIRA_CONFIG: fourth argument must be a Regexp.\n";
+    } else {
+	$MatchLog = qr/[A-Z]{2,}/;
+    }
+
+    @_ >= 3 && @_ <= 5
+	or croak "CHECK_JIRA_CONFIG: requires three, four, or five arguments.\n";
+
+    $BaseURL =~ s/\/+$//;
+
+    return 1;
+}
+
+
+sub _validate_projects {
+    my ($opt, $val) = @_;
+    is_string($val) && $val =~ /^[A-Z,]+$/
+	or croak "$HOOK: $opt\'s value must be a string matching /^[A-Z,]+\$/.\n";
+    my %projects = map {$_ => undef} grep {/./} split /,/, $val;
+    return \%projects;
+}
+
+sub _validate_bool {
+    my ($opt, $val) = @_;
+    defined $val or croak "$HOOK: undefined $opt\'s value.\n";
+    return $val;
+}
+
+sub _validate_code {
+    my ($opt, $val) = @_;
+    is_code_ref($val) or croak "$HOOK: $opt\'s value must be a CODE-ref.\n";
+    return $val;
+}
+
+sub _validate_regex {
+    my ($opt, $val) = @_;
+    is_rx($val) or croak "$HOOK: $opt\'s value must be a qr/REGEX/.\n";
+    return $val;
+}
+
+my %opt_checks = (
+    projects          => \&_validate_projects,
+    require           => \&_validate_bool,
+    valid             => \&_validate_bool,
+    unresolved        => \&_validate_bool,
+    by_assignee       => \&_validate_bool,
+    check_one         => \&_validate_code,
+    check_all         => \&_validate_code,
+    check_all_svnlook => \&_validate_code,
+    post_action       => \&_validate_code,
+    exclude           => \&_validate_regex,
+);
+
+sub CHECK_JIRA {
+    my ($regex, $opts) = @_;
+    is_rx($regex) || (is_string($regex) && $regex eq 'default')
+	or croak "$HOOK: first arg must be a qr/Regexp/ or the string 'default'.\n";
+    ! defined $opts || is_hash_ref($opts)
+	or croak "$HOOK: second argument must be a HASH-ref.\n";
+
+    $opts = {} unless defined $opts;
+    foreach my $opt (keys %$opts) {
+	exists $opt_checks{$opt} or croak "$HOOK: unknown option '$opt'.\n";
+	$opts->{$opt} = $opt_checks{$opt}->($opt, $opts->{$opt});
+    }
+
+    if (ref $regex) {
+	push @Checks, [$regex => $opts];
+    } else {
+	while (my ($opt, $val) = each %$opts) {
+	    $Defaults{$opt} = $val;
+	}
+    }
+    PRE_COMMIT(\&pre_commit);
+    POST_COMMIT(\&post_commit) if exists $opts->{post_action};
+
+    return 1;
+}
+
+sub _pre_checks {
+    my ($svnlook, $keys, $opts) = @_;
+
+    # Grok and check each JIRA issue
+    my @issues;
+    foreach my $key (@$keys) {
+	my $issue = eval {$JIRA->getIssue($key)};
+	if ($opts->{valid}) {
+	    croak "$HOOK: issue $key is not valid: $@\n" if $@;
+	}
+	$issue or next;
+	if ($opts->{unresolved}) {
+	    croak "$HOOK: issue $key is already resolved.\n"
+		if defined $issue->{resolution};
+	}
+	if ($opts->{by_assignee}) {
+	    my $author = $svnlook->author();
+	    croak "$HOOK: committer ($author) is different from issue ${key}'s assignee ($issue->{assignee}).\n"
+		if $author ne $issue->{assignee};
+	}
+	if (my $check = $opts->{check_one}) {
+	    $check->($JIRA, $issue, $svnlook);
+	}
+	push @issues, $issue;
+    }
+
+    if (my $check = $opts->{check_all}) {
+	$check->($JIRA, @issues) if @issues;
+    }
+
+    if (my $check = $opts->{check_all_svnlook}) {
+	$check->($svnlook, $JIRA, @issues) if @issues;
+    }
+
+    return;
+}
+
+sub _post_action {
+    my ($svnlook, $keys, $opts) = @_;
+
+    if (my $action = $opts->{post_action}) {
+	$action->($JIRA, $svnlook, @$keys);
+    }
+
+    return;
+}
+
+sub _check_if_needed {
+    my ($svnlook, $docheck) = @_;
+
+    defined $BaseURL
+	or croak "$HOOK: plugin not configured. Please, use the CHECK_JIRA_CONFIG directive.\n";
+
+    my @files = $svnlook->changed();
+
+    foreach my $check (@Checks) {
+	my ($regex, $opts) = @$check;
+
+	for my $file (@files) {
+	    if ($file =~ $regex) {
+		# skip exclusions
+		next if exists $opts->{exclude} && $file =~ $opts->{exclude};
+
+		# Grok the JIRA issue keys from the commit log
+		my ($match) = ($svnlook->log_msg() =~ $MatchLog);
+		my @keys    = defined $match ? $match =~ /\b$MatchKey-\d+\b/g : ();
+
+		my %opts = (%Defaults, %$opts);
+
+		if ($opts{require}) {
+		    croak "$HOOK: you must cite at least one JIRA issue key in the commit message.\n"
+			unless @keys;
+		}
+
+		return unless @keys;
+
+		# Check if there is a restriction on the project keys allowed
+		if (exists $opts->{projects}) {
+		    foreach my $key (@keys) {
+			my ($pkey, $pnum) = split /-/, $key;
+			croak "$HOOK: issue $key is not allowed. You must cite only JIRA issues for the following projects: ", join(', ', sort keys %{$opts->{projects}}), ".\n"
+			    unless exists $opts->{projects}{$pkey};
+		    }
+		}
+
+		# Connect to JIRA if not yet connected.
+		unless (defined $JIRA) {
+		    $JIRA = eval {JIRA::Client->new($BaseURL, $Login, $Passwd)};
+		    croak "CHECK_JIRA_CONFIG: cannot connect to the JIRA server: $@\n" if $@;
+		}
+
+		$docheck->($svnlook, \@keys, \%opts);
+		last;
+	    }
+	}
+    }
+
+    return;
+}
+
+sub pre_commit {
+    my ($svnlook) = @_;
+    _check_if_needed($svnlook, \&_pre_checks);
+    return;
+}
+
+sub post_commit {
+    my ($svnlook) = @_;
+    _check_if_needed($svnlook, \&_post_action);
+    return;
+}
+
+1; # End of SVN::Hooks::CheckJira
+
+__END__
+=pod
 
 =head1 NAME
 
 SVN::Hooks::CheckJira - Integrate Subversion with the JIRA ticketing system.
 
+=head1 VERSION
+
+version 1.19
+
 =head1 DESCRIPTION
 
 This SVN::Hooks plugin requires that any Subversion commits affecting
@@ -58,43 +283,6 @@ The JIRA issue keys are extracted from the commit log (or the part of
 it specified by the REGEXP) with the following pattern:
 C<qr/\b([A-Z]+-\d+)\b/g>;
 
-=cut
-
-my ($BaseURL, $Login, $Passwd, $MatchLog, $MatchKey);
-my $JIRA;
-my @Checks;
-my %Defaults = (
-    require     => 1,
-    valid       => 1,
-    unresolved  => 1,
-    by_assignee => 0,
-);
-
-sub CHECK_JIRA_CONFIG {
-    ($BaseURL, $Login, $Passwd, $MatchLog, $MatchKey) = @_;
-
-    if (defined $MatchKey) {
-	ref $MatchKey && ref $MatchKey eq 'Regexp'
-	    or croak "CHECK_JIRA_CONFIG: fifth argument must be a Regexp.\n";
-    } else {
-	$MatchKey = qr/[A-Z]{2,}/;
-    }
-
-    if (defined $MatchLog) {
-	ref $MatchLog && ref $MatchLog eq 'Regexp'
-	    or croak "CHECK_JIRA_CONFIG: fourth argument must be a Regexp.\n";
-    } else {
-	$MatchLog = qr/[A-Z]{2,}/;
-    }
-
-    @_ >= 3 && @_ <= 5
-	or croak "CHECK_JIRA_CONFIG: requires three, four, or five arguments.\n";
-
-    $BaseURL =~ s/\/+$//;
-
-    return 1;
-}
-
 =head2 CHECK_JIRA(REGEXP => {OPT => VALUE, ...})
 
 This directive tells how each part of the repository structure must be
@@ -190,6 +378,14 @@ information about the commit proper.  The following arguments are the
 JIRA keys mentioned in the commit log message. The value returned by
 the routine, if any, is ignored.
 
+=item exclude => REGEXP
+
+Normally you specify a CHECK_JIRA with a regex matching a root
+directory in the repository hierarchy. Sometimes you need to specify
+some subparts of that root directory that shouldn't be treated by this
+CHECK_JIRA directive. You can use this option to specify these
+exclusions by means of another regex.
+
 =back
 
 You can set defaults for these options using a CHECK_JIRA directive
@@ -248,227 +444,26 @@ add a comment to each refered issue like this:
         post_action => add_comment("Subversion Commit r{rev} by {author} on {date}\n{log_msg}")
     });
 
-=cut
+You can use a generic CHECK_JIRA excluding specific directories from
+it using the "exclude" option like this:
 
-sub _validate_projects {
-    my ($opt, $val) = @_;
-    defined $val         or croak "$HOOK: undefined $opt\'s value.\n";
-    ref $val            and croak "$HOOK: $opt\'s value must be a scalar.\n";
-    $val =~ /^[A-Z,]+$/  or croak "$HOOK: $opt\'s value must match /^[A-Z,]+\$/.\n";
-    my %projects = map {$_ => undef} grep {/./} split /,/, $val;
-    return \%projects;
-}
+    CHECK_JIRA(qr:^(trunk|branches/[^/]): => {
+        exclude => qr:/documentation/:,
+        # other options...
+    });
 
-sub _validate_bool {
-    my ($opt, $val) = @_;
-    defined $val or croak "$HOOK: undefined $opt\'s value.\n";
-    return $val;
-}
-
-sub _validate_code {
-    my ($opt, $val) = @_;
-    ref $val and ref $val eq 'CODE'
-	or croak "$HOOK: $opt\'s value must be a CODE-ref.\n";
-    return $val;
-}
-
-my %opt_checks = (
-    projects          => \&_validate_projects,
-    require           => \&_validate_bool,
-    valid             => \&_validate_bool,
-    unresolved        => \&_validate_bool,
-    by_assignee       => \&_validate_bool,
-    check_one         => \&_validate_code,
-    check_all         => \&_validate_code,
-    check_all_svnlook => \&_validate_code,
-    post_action       => \&_validate_code,
-);
-
-sub CHECK_JIRA {
-    my ($regex, $opts) = @_;
-    croak "$HOOK: first arg must be a qr/Regexp/ or the string 'default'.\n"
-	unless (ref $regex and ref $regex eq 'Regexp') or (not ref $regex and $regex eq 'default');
-    croak "$HOOK: second argument must be a HASH-ref.\n"
-	if defined $opts and not (ref $opts and ref $opts eq 'HASH');
-
-    $opts = {} unless defined $opts;
-    foreach my $opt (keys %$opts) {
-	exists $opt_checks{$opt} or croak "$HOOK: unknown option '$opt'.\n";
-	$opts->{$opt} = $opt_checks{$opt}->($opt, $opts->{$opt});
-    }
-
-    if (ref $regex) {
-	push @Checks, [$regex => $opts];
-    }
-    else {
-	while (my ($opt, $val) = each %$opts) {
-	    $Defaults{$opt} = $val;
-	}
-    }
-    PRE_COMMIT(\&pre_commit);
-    POST_COMMIT(\&post_commit) if exists $opts->{post_action};
-
-    return 1;
-}
-
-sub _pre_checks {
-    my ($svnlook, $keys, $opts) = @_;
-
-    # Grok and check each JIRA issue
-    my @issues;
-    foreach my $key (@$keys) {
-	my $issue = eval {$JIRA->getIssue($key)};
-	if ($opts->{valid}) {
-	    croak "$HOOK: issue $key is not valid: $@\n" if $@;
-	}
-	$issue or next;
-	if ($opts->{unresolved}) {
-	    croak "$HOOK: issue $key is already resolved.\n"
-		if defined $issue->{resolution};
-	}
-	if ($opts->{by_assignee}) {
-	    my $author = $svnlook->author();
-	    croak "$HOOK: committer ($author) is different from issue ${key}'s assignee ($issue->{assignee}).\n"
-		if $author ne $issue->{assignee};
-	}
-	if (my $check = $opts->{check_one}) {
-	    $check->($JIRA, $issue, $svnlook);
-	}
-	push @issues, $issue;
-    }
-
-    if (my $check = $opts->{check_all}) {
-	$check->($JIRA, @issues) if @issues;
-    }
-
-    if (my $check = $opts->{check_all_svnlook}) {
-	$check->($svnlook, $JIRA, @issues) if @issues;
-    }
-
-    return;
-}
-
-sub _post_action {
-    my ($svnlook, $keys, $opts) = @_;
-
-    if (my $action = $opts->{post_action}) {
-	$action->($JIRA, $svnlook, @$keys);
-    }
-
-    return;
-}
-
-sub _check_if_needed {
-    my ($svnlook, $docheck) = @_;
-
-    defined $BaseURL
-	or croak "$HOOK: plugin not configured. Please, use the CHECK_JIRA_CONFIG directive.\n";
-
-    my @files = $svnlook->changed();
-
-    foreach my $check (@Checks) {
-	my ($regex, $opts) = @$check;
-
-	for my $file (@files) {
-	    if ($file =~ $regex) {
-
-		# Grok the JIRA issue keys from the commit log
-		my ($match) = ($svnlook->log_msg() =~ $MatchLog);
-		my @keys    = defined $match ? $match =~ /\b$MatchKey-\d+\b/g : ();
-
-		my %opts = (%Defaults, %$opts);
-
-		if ($opts{require}) {
-		    croak "$HOOK: you must cite at least one JIRA issue key in the commit message.\n"
-			unless @keys;
-		}
-
-		return unless @keys;
-
-		# Check if there is a restriction on the project keys allowed
-		if (exists $opts->{projects}) {
-		    foreach my $key (@keys) {
-			my ($pkey, $pnum) = split /-/, $key;
-			croak "$HOOK: issue $key is not allowed. You must cite only JIRA issues for the following projects: ", join(', ', sort keys %{$opts->{projects}}), ".\n"
-			    unless exists $opts->{projects}{$pkey};
-		    }
-		}
-
-		# Connect to JIRA if not yet connected.
-		unless (defined $JIRA) {
-		    $JIRA = eval {JIRA::Client->new($BaseURL, $Login, $Passwd)};
-		    croak "CHECK_JIRA_CONFIG: cannot connect to the JIRA server: $@\n" if $@;
-		}
-
-		$docheck->($svnlook, \@keys, \%opts);
-		last;
-	    }
-	}
-    }
-
-    return;
-}
-
-sub pre_commit {
-    my ($svnlook) = @_;
-    _check_if_needed($svnlook, \&_pre_checks);
-    return;
-}
-
-sub post_commit {
-    my ($svnlook) = @_;
-    _check_if_needed($svnlook, \&_post_action);
-    return;
-}
+=for Pod::Coverage post_commit pre_commit
 
 =head1 AUTHOR
 
-Gustavo Chaves, C<< <gnustavo@cpan.org> >>
+Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
-=head1 BUGS
+=head1 COPYRIGHT AND LICENSE
 
-Please report any bugs or feature requests to
-C<bug-svn-hooks at rt.cpan.org>, or through the web
-interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=SVN-Hooks>.  I will
-be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
+This software is copyright (c) 2012 by CPqD.
 
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc SVN::Hooks
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=SVN-Hooks>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/SVN-Hooks>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/SVN-Hooks>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/SVN-Hooks>
-
-=back
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2009-2011 CPqD, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
-1; # End of SVN::Hooks::CheckJira
